@@ -50,7 +50,7 @@ static UInitOnce gSpecialInversesInitOnce = U_INITONCE_INITIALIZER;
 /**
  * The mutex controlling access to SPECIAL_INVERSES
  */
-static UMutex LOCK = U_MUTEX_INITIALIZER;
+static UMutex LOCK;
 
 TransliteratorIDParser::Specs::Specs(const UnicodeString& s, const UnicodeString& t,
                                      const UnicodeString& v, UBool sawS,
@@ -294,6 +294,7 @@ UnicodeSet* TransliteratorIDParser::parseGlobalFilter(const UnicodeString& id, i
         pos = ppos.getIndex();
 
         if (withParens == 1 && !ICU_Utility::parseChar(id, pos, CLOSE_REV)) {
+            delete filter;
             pos = start;
             return NULL;
         }
@@ -363,6 +364,8 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
     int32_t pos = 0;
     int32_t withParens = 1;
     list.removeAllElements();
+    UObjectDeleter *save = list.setDeleter(_deleteSingleID);
+
     UnicodeSet* filter;
     globalFilter = NULL;
     canonID.truncate(0);
@@ -391,7 +394,7 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
             break;
         }
         if (dir == FORWARD) {
-            list.addElement(single, ec);
+            list.adoptElement(single, ec);
         } else {
             list.insertElementAt(single, 0, ec);
         }
@@ -441,10 +444,10 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
         goto FAIL;
     }
 
+    list.setDeleter(save);
     return TRUE;
 
  FAIL:
-    UObjectDeleter *save = list.setDeleter(_deleteSingleID);
     list.removeAllElements();
     list.setDeleter(save);
     delete globalFilter;
@@ -493,9 +496,8 @@ void TransliteratorIDParser::instantiateList(UVector& list,
                 ec = U_INVALID_ID;
                 goto RETURN;
             }
-            tlist.addElement(t, ec);
+            tlist.adoptElement(t, ec);
             if (U_FAILURE(ec)) {
-                delete t;
                 goto RETURN;
             }
         }
@@ -508,10 +510,7 @@ void TransliteratorIDParser::instantiateList(UVector& list,
             // Should never happen
             ec = U_INTERNAL_TRANSLITERATOR_ERROR;
         }
-        tlist.addElement(t, ec);
-        if (U_FAILURE(ec)) {
-            delete t;
-        }
+        tlist.adoptElement(t, ec);
     }
 
  RETURN:
@@ -524,9 +523,8 @@ void TransliteratorIDParser::instantiateList(UVector& list,
 
         while (tlist.size() > 0) {
             t = (Transliterator*) tlist.orphanElementAt(0);
-            list.addElement(t, ec);
+            list.adoptElement(t, ec);
             if (U_FAILURE(ec)) {
-                delete t;
                 list.removeAllElements();
                 break;
             }
