@@ -1,4 +1,4 @@
-// Copyright (C) 2016 and later: Unicode, Inc. and others.
+// © 2016 and later: Unicode, Inc. and others.
 // License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
@@ -85,7 +85,7 @@ void Test4029195()
     if(fmdt) {
       log_verbose("today: %s\n", austrdup(fmdt));
     } else {
-      log_data_err("ERROR: couldn't format, exitting test");
+      log_data_err("ERROR: couldn't format, exiting test");
       return;
     }
     
@@ -166,6 +166,7 @@ void Test4056591()
     if(U_FAILURE(status))
     {
         log_data_err("FAIL: error in creating the dateformat using u_openPattern(): %s - (Are you missing data?)\n", myErrorName(status));
+        ucal_close(cal);
         return;
     }
     start = 1800;
@@ -350,7 +351,7 @@ void Test4061287()
     if(U_FAILURE(status))
         ok = TRUE;
     if(ok!=TRUE) 
-        log_err("Fail: Lenient not working: does lenient parsing in spite of setting Leninent as FALSE ");
+        log_err("Fail: Lenient not working: does lenient parsing in spite of setting Lenient as FALSE ");
 
     udat_close(df);
     
@@ -423,7 +424,7 @@ void Test4073003()
         result =myFormatit(fmt, d);
         result2 =myFormatit(fmt, dd);
         if(!result || !result2) {
-            log_data_err("Fail: could not format - exitting test\n");
+            log_data_err("Fail: could not format - exiting test\n");
             return;
         }
         if (u_strcmp(result, result2)!=0){
@@ -490,7 +491,7 @@ void Test714(void)
     }
     result =myFormatit(fmt, d);
     if(!result) {
-      log_data_err("Fail: could not format - exitting test\n");
+      log_data_err("Fail: could not format - exiting test\n");
       return;
     }
     u_uastrcpy(temp, expect);
