@@ -569,10 +569,10 @@ isAcceptable1(void *context,
         pInfo->dataFormat[3]==0x6c &&
         pInfo->formatVersion[0]==3 )
     {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable1()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable1()\n", name, type);
         return TRUE;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable1():-\n"
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable1():-\n"
             "\tsize              = %d\n"
             "\tisBigEndian       = %d\n"
             "\tcharsetFamily     = %d\n"
@@ -582,7 +582,7 @@ isAcceptable1(void *context,
             name, type, pInfo->size,  pInfo->isBigEndian, pInfo->charsetFamily, pInfo->formatVersion[0], 
             pInfo->dataVersion[0], pInfo->dataFormat[0], pInfo->dataFormat[1], pInfo->dataFormat[2], 
             pInfo->dataFormat[3]);  
-        log_verbose("Call another verifing function to accept the data\n");
+        log_verbose("Call another verifying function to accept the data\n");
         return FALSE;
     }
 }
@@ -606,10 +606,10 @@ isAcceptable2(void *context,
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==unicodeVersion[0] )
     {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable2()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable2()\n", name, type);
         return TRUE;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable2()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable2()\n", name, type);
 
         return FALSE;
     }
@@ -631,11 +631,11 @@ isAcceptable3(void *context,
         pInfo->dataFormat[3]==0x74 &&
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==1   ) {
-        log_verbose("The data from \"%s.%s\" IS acceptable using the verifing function isAcceptable3()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable3()\n", name, type);
 
         return TRUE;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable3()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable3()\n", name, type);
         return FALSE;
     }
 
@@ -731,11 +731,11 @@ isAcceptable(void *context,
         pInfo->formatVersion[0]==1 &&
         pInfo->dataVersion[0]==1   &&
         *((int*)context) == 2 ) {
-        log_verbose("The data from\"%s.%s\" IS acceptable using the verifing function isAcceptable()\n", name, type);
+        log_verbose("The data from\"%s.%s\" IS acceptable using the verifying function isAcceptable()\n", name, type);
 
         return TRUE;
     } else {
-        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifing function isAcceptable()\n", name, type);
+        log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable()\n", name, type);
         return FALSE;
     }
 }
@@ -1353,7 +1353,7 @@ static const struct {
 
 #if !UCONFIG_NO_BREAK_ITERATION
     {"char",                     "brk", ubrk_swap},
-    {"thaidict",                 "dict",udict_swap},
+    {"laodict",                  "dict",udict_swap},
 #endif
 
 #if 0
@@ -1857,7 +1857,7 @@ static void TestTZDataDir(void) {
     //     whatever/.../testdata/out/testdata
     // The test data puts an old (2014a) version of the time zone data there.
 
-    // Switch ICU to the testdata version of zoneinfo64.res, which is verison 2014a.
+    // Switch ICU to the testdata version of zoneinfo64.res, which is version 2014a.
     ctest_resetICU();
     u_setTimeZoneFilesDirectory(testDataPath, &status);
     tzDataVersion = ucal_getTZDataVersion(&status);
