@@ -45,6 +45,7 @@ void IntlTestDateTimePatternGeneratorAPI::runIndexedTest( int32_t index, UBool e
         TESTCASE(9, testFallbackWithDefaultRootLocale);
         TESTCASE(10, testGetDefaultHourCycle_OnEmptyInstance);
         TESTCASE(11, test_jConsistencyOddLocales);
+        TESTCASE(12, testBestPattern);
         default: name = ""; break;
     }
 }
@@ -252,13 +253,13 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         CharsToUnicodeString("1999\\u5E74\\u7B2C1\\u5B63\\u5EA6"),        // 07: yQQQ
         CharsToUnicodeString("\\u4E0B\\u534811:58"),                      // 08: hhmm
         UnicodeString("23:58"),                                           // 09: HHmm
-        CharsToUnicodeString("\\u4E0B\\u534811:58"),                      // 10: jjmm
+        CharsToUnicodeString("23:58"),                                    // 10: jjmm
         UnicodeString("58:59"),                                           // 11: mmss
         CharsToUnicodeString("1999\\u5E741\\u6708"),                      // 12: yyyyMMMM  -> yyyy\u5E74MMM
         CharsToUnicodeString("1\\u670813\\u65E5\\u5468\\u4E09"),          // 13: MMMEd -> MMMd\u65E5EEE
         CharsToUnicodeString("13\\u65E5\\u5468\\u4E09"),                  // 14: Ed    -> d\u65E5EEE
-        CharsToUnicodeString("\\u4E0B\\u534811:58:59.123"),               // 15: jmmssSSS -> "ah:mm:ss.SSS"
-        UnicodeString("11:58"),                                           // 16: JJmm
+        CharsToUnicodeString("23:58:59.123"),                             // 15: jmmssSSS -> "ah:mm:ss.SSS"
+        UnicodeString("23:58"),                                           // 16: JJmm
     };
 
     UnicodeString patternResults_zh_TW_roc[] = {
@@ -315,13 +316,13 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         CharsToUnicodeString("1998\\u620A\\u5BC5\\u5E74\\u7b2c\\u56db\\u5B63\\u5EA6"),  // 07: yQQQ
         CharsToUnicodeString("\\u4E0B\\u534811:58"),                            // 08: hhmm
         UnicodeString("23:58"),                                                 // 09: HHmm
-        CharsToUnicodeString("\\u4E0B\\u534811:58"),                            // 10: jjmm
+        CharsToUnicodeString("23:58"),                                          // 10: jjmm
         UnicodeString("58:59"),                                                 // 11: mmss
         CharsToUnicodeString("1998\\u620A\\u5BC5\\u5E74\\u5341\\u4E00\\u6708"), // 12: yyyyMMMM
         CharsToUnicodeString("\\u5341\\u4E00\\u670826\\u65E5\\u5468\\u4E09"),   // 13: MMMEd
         CharsToUnicodeString("26\\u65E5\\u5468\\u4E09"),                        // 14: Ed    -> d\u65E5EEE
-        CharsToUnicodeString("\\u4E0B\\u534811:58:59.123"),                     // 15: jmmssSS
-        UnicodeString("11:58"),                                                 // 16: JJmm
+        CharsToUnicodeString("23:58:59.123"),                                   // 15: jmmssSS
+        UnicodeString("23:58"),                                                 // 16: JJmm
     };
 
     UnicodeString patternResults_ja_jp_traditional[] = {
@@ -444,7 +445,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
 
     DateTimePatternGenerator *instFromDefaultLocale=DateTimePatternGenerator::createInstance(status);
     if (U_FAILURE(status)) {
-        dataerrln("ERROR: Could not create DateTimePatternGenerator (default) - exitting");
+        dataerrln("ERROR: Could not create DateTimePatternGenerator (default) - exiting");
         return;
     }
     else {
@@ -456,7 +457,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
     status = U_ZERO_ERROR;
     DateTimePatternGenerator *instFromLocale=DateTimePatternGenerator::createInstance(Locale::getFrench(), status);
     if (U_FAILURE(status)) {
-        dataerrln("ERROR: Could not create DateTimePatternGenerator (Locale::getFrench()) - exitting");
+        dataerrln("ERROR: Could not create DateTimePatternGenerator (Locale::getFrench()) - exiting");
         return;
     }
 
@@ -492,7 +493,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
     UDate sampleDate=LocaleTest::date(99, 9, 13, 23, 58, 59);
     DateTimePatternGenerator *gen = DateTimePatternGenerator::createInstance(deLocale, status);
     if (U_FAILURE(status)) {
-        dataerrln("ERROR: Could not create DateTimePatternGenerator (Locale::getGermany()) - exitting");
+        dataerrln("ERROR: Could not create DateTimePatternGenerator (Locale::getGermany()) - exiting");
         return;
     }
     UnicodeString findPattern = gen->getBestPattern(UnicodeString("MMMddHmm"), status);
@@ -701,7 +702,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         logln("\n\n Locale: %s_%s_%s@%s", testLocale[localeIndex][0], testLocale[localeIndex][1], testLocale[localeIndex][2], testLocale[localeIndex][3]);
         DateTimePatternGenerator *patGen=DateTimePatternGenerator::createInstance(loc, status);
         if(U_FAILURE(status)) {
-            dataerrln("ERROR: Could not create DateTimePatternGenerator with locale index:%d . - exitting\n", localeIndex);
+            dataerrln("ERROR: Could not create DateTimePatternGenerator with locale index:%d . - exiting\n", localeIndex);
             return;
         }
         while (patternData[dataIndex].length() > 0) {
@@ -739,7 +740,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         logln("\n\n Test various skeletons for English locale...");
         DateTimePatternGenerator *patGen=DateTimePatternGenerator::createInstance(Locale::getEnglish(), status);
         if(U_FAILURE(status)) {
-            dataerrln("ERROR: Could not create DateTimePatternGenerator with locale English . - exitting\n");
+            dataerrln("ERROR: Could not create DateTimePatternGenerator with locale English . - exiting\n");
             return;
         }
         TimeZone *enZone = TimeZone::createTimeZone(UnicodeString("ECT/GMT"));
@@ -776,7 +777,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
     // ======= Test random skeleton
     DateTimePatternGenerator *randDTGen= DateTimePatternGenerator::createInstance(status);
     if (U_FAILURE(status)) {
-        dataerrln("ERROR: Could not create DateTimePatternGenerator (Locale::getFrench()) - exitting");
+        dataerrln("ERROR: Could not create DateTimePatternGenerator (Locale::getFrench()) - exiting");
         return;
     }
     UChar newChar;
@@ -811,7 +812,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
 
     test = DateTimePatternGenerator::createEmptyInstance(status);
     if(U_FAILURE(status)) {
-         errln("ERROR: Fail to create an empty instance ! - exitting.\n");
+         errln("ERROR: Fail to create an empty instance ! - exiting.\n");
          delete test;
          return;
     }
@@ -949,8 +950,8 @@ void IntlTestDateTimePatternGeneratorAPI::testOptions(/*char *par*/)
         { "en@calendar=chinese",  "Gy",    "r(U)",     UDATPG_MATCH_NO_OPTIONS },
         { "en@calendar=chinese",  "GU",    "r(U)",     UDATPG_MATCH_NO_OPTIONS },
         { "en@calendar=chinese",  "ULLL",  "MMM U",    UDATPG_MATCH_NO_OPTIONS },
-        { "en@calendar=chinese",  "yMMM",  "MMM r(U)", UDATPG_MATCH_NO_OPTIONS },
-        { "en@calendar=chinese",  "GUMMM", "MMM r(U)", UDATPG_MATCH_NO_OPTIONS },
+        { "en@calendar=chinese",  "yMMM",  "MMM r",    UDATPG_MATCH_NO_OPTIONS },
+        { "en@calendar=chinese",  "GUMMM", "MMM r",    UDATPG_MATCH_NO_OPTIONS },
         { "zh@calendar=chinese",  "yyyy",  "rU\\u5E74",    UDATPG_MATCH_NO_OPTIONS },
         { "zh@calendar=chinese",  "YYYY",  "YY\\u5E74",    UDATPG_MATCH_NO_OPTIONS }, // not a good result, may want r(Y) or r(U)
         { "zh@calendar=chinese",  "U",     "rU\\u5E74",    UDATPG_MATCH_NO_OPTIONS },
@@ -1159,12 +1160,12 @@ void IntlTestDateTimePatternGeneratorAPI::testStaticGetSkeleton(/*char *par*/)
 void IntlTestDateTimePatternGeneratorAPI::testC() {
     const char* tests[][3] = {
             // These may change with actual data for Bhmm/bhmm skeletons
-            {"zh",     "Cm",      "Bh:mm"},
-            {"zh",     "CCm",     "Bhh:mm"},
-            {"zh",     "CCCm",    "BBBBh:mm"},
-            {"zh",     "CCCCm",   "BBBBhh:mm"},
-            {"zh",     "CCCCCm",  "BBBBBh:mm"},
-            {"zh",     "CCCCCCm", "BBBBBhh:mm"},
+            {"zh-TW",  "Cm",      "Bh:mm"},
+            {"zh-TW",  "CCm",     "Bhh:mm"},
+            {"zh-TW",  "CCCm",    "BBBBh:mm"},
+            {"zh-TW",  "CCCCm",   "BBBBhh:mm"},
+            {"zh-TW",  "CCCCCm",  "BBBBBh:mm"},
+            {"zh-TW",  "CCCCCCm", "BBBBBhh:mm"},
             {"de",     "Cm",      "HH:mm"},
             {"de",     "CCm",     "HH:mm"},
             {"de",     "CCCm",    "HH:mm"},
@@ -1486,7 +1487,7 @@ void IntlTestDateTimePatternGeneratorAPI::testFallbackWithDefaultRootLocale() {
     }
 }
 
-// ICU-21000 Ensure that calling getDefaultHourCycle on an empty instance doesn't call UPRV_UNREACHABLE/abort.
+// ICU-21000 Ensure that calling getDefaultHourCycle on an empty instance doesn't call UPRV_UNREACHABLE_EXIT/abort.
 void IntlTestDateTimePatternGeneratorAPI::testGetDefaultHourCycle_OnEmptyInstance() {
     UErrorCode status = U_ZERO_ERROR;
 
@@ -1557,4 +1558,77 @@ void IntlTestDateTimePatternGeneratorAPI::test_jConsistencyOddLocales() { // ICU
     }
 }
 
+void IntlTestDateTimePatternGeneratorAPI::testBestPattern() {
+    // generic test for DateTimePatternGenerator::getBestPattern() that can be used to test multiple
+    // bugs in the resource data
+    const struct TestCase {
+        const char* localeID;
+        const char* skeleton;
+        const UChar* expectedPattern;
+    } testCases[] = {
+        // ICU-21650: (See the "week day" section of https://www.unicode.org/reports/tr35/tr35-dates.html#Date_Field_Symbol_Table
+        // for a full explanation of why this is the desired behavior)
+        // if the user asks for E, the minimum field length is 3, but if he asks for c or e, it's 1
+        { "en_US",      "E",           u"ccc"    },
+        { "en_US",      "c",           u"c"      },
+        { "en_US",      "e",           u"c"      },
+        { "en_US",      "EE",          u"ccc"    },
+        { "en_US",      "cc",          u"cc"     },
+        { "en_US",      "ee",          u"cc"     },
+        { "en_US",      "EEE",         u"ccc"    },
+        { "en_US",      "ccc",         u"ccc"    },
+        { "en_US",      "eee",         u"ccc"    },
+        // and if the user asked for c or e and the field length is 1 or 2, the output pattern should contain
+        // e instead of E (e supports numeric abbreviations; E doesn't)
+        { "en_US",      "yMEd",        u"EEE, M/d/y"         },
+        { "en_US",      "yMcd",        u"e, M/d/y"           },
+        { "en_US",      "yMed",        u"e, M/d/y"           },
+        { "en_US",      "yMMEEdd",     u"EEE, MM/dd/y"       },
+        { "en_US",      "yMMccdd",     u"ee, MM/dd/y"        },
+        { "en_US",      "yMMeedd",     u"ee, MM/dd/y"        },
+        { "en_US",      "yMMMEd",      u"EEE, MMM d, y"      },
+        { "en_US",      "yMMMcccd",    u"EEE, MMM d, y"      },
+        { "en_US",      "yMMMeeed",    u"EEE, MMM d, y"      },
+        { "en_US",      "yMMMMEEEEd",  u"EEEE, MMMM d, y"    },
+        { "en_US",      "yMMMMccccd",  u"EEEE, MMMM d, y"    },
+        { "en_US",      "yMMMMeeeed",  u"EEEE, MMMM d, y"    },
+        // ICU-21428: Bad patterns for nonstandard calendars
+        { "en_GB",                   "yMd", u"dd/MM/y"          },
+        { "en_GB@calendar=coptic",   "yMd", u"dd/MM/y GGGGG"    },
+        { "en_GB@calendar=japanese", "yMd", u"dd/MM/y GGGGG"    },
+        { "en_GB@calendar=buddhist", "yMd", u"dd/MM/y GGGGG"    },
+        // ICU-20992: Bad patterns for missing fields
+        { "ckb_IR",     "mmSSS",       u"mm:ss\u066bSSS"     },
+        { "ckb_IR",     "BSSS",        u"SSS \u251c'Dayperiod': B\u2524" },
+    };
+    
+    for (int32_t i = 0; i < UPRV_LENGTHOF(testCases); i++) {
+        UErrorCode err = U_ZERO_ERROR;
+        UnicodeString actualPattern;
+        
+        if (uprv_strcmp(testCases[i].skeleton, "full") != 0) {
+            LocalPointer<DateTimePatternGenerator> dtpg(DateTimePatternGenerator::createInstance(testCases[i].localeID, err), err);
+            actualPattern = dtpg->getBestPattern(UnicodeString(testCases[i].skeleton), err);
+        } else {
+            LocalPointer<DateFormat> df(DateFormat::createDateInstance(DateFormat::kFull, testCases[i].localeID));
+            SimpleDateFormat* sdf = dynamic_cast<SimpleDateFormat*>(df.getAlias());
+            
+            if (sdf != NULL) {
+                sdf->toPattern(actualPattern);
+            }
+        }
+        
+        if (U_FAILURE(err)) {
+            errln("Failure for test case %s/%s: %s", testCases[i].localeID, testCases[i].skeleton, u_errorName(err));
+        } else {
+            char failureMessage[100];
+            strcpy(failureMessage, "Wrong result for test case ");
+            strcat(failureMessage, testCases[i].localeID);
+            strcat(failureMessage, "/");
+            strcat(failureMessage, testCases[i].skeleton);
+            assertEquals(failureMessage, testCases[i].expectedPattern, actualPattern);
+        }
+    }
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
