@@ -30,7 +30,7 @@ public:
 
     CollationMonkeyTest();
     virtual ~CollationMonkeyTest();
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ) override;
 
     // utility function used in tests, returns absolute value
     int32_t checkValue(int32_t value);
