@@ -42,7 +42,7 @@ class AffixUtilsTest : public IntlTest {
     void testInvalid();
     void testUnescapeWithSymbolProvider();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     UnicodeString unescapeWithDefaults(const SymbolProvider &defaultProvider, UnicodeString input,
@@ -59,17 +59,22 @@ class NumberFormatterApiTest : public IntlTestWithFieldPosition {
     void notationCompact();
     void unitMeasure();
     void unitCompoundMeasure();
+    void unitArbitraryMeasureUnits();
     void unitSkeletons();
     void unitUsage();
     void unitUsageErrorCodes();
     void unitUsageSkeletons();
     void unitCurrency();
+    void unitInflections();
+    void unitGender();
+    void unitNotConvertible();
     void unitPercent();
     void percentParity();
     void roundingFraction();
     void roundingFigures();
     void roundingFractionFigures();
     void roundingOther();
+    void roundingIncrementRegressionTest();
     void grouping();
     void padding();
     void integerWidth();
@@ -95,7 +100,7 @@ class NumberFormatterApiTest : public IntlTestWithFieldPosition {
     void toDecimalNumber();
     void microPropsInternals();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     CurrencyUnit USD;
@@ -170,6 +175,20 @@ class NumberFormatterApiTest : public IntlTestWithFieldPosition {
       const FormattedNumber& formattedNumber,
       const UFieldPosition* expectedFieldPositions,
       int32_t length);
+
+    struct UnitInflectionTestCase {
+        const char *unitIdentifier;
+        const char *locale;
+        const char *unitDisplayCase;
+        double value;
+        const UChar *expected;
+    };
+
+    void runUnitInflectionsTestCases(UnlocalizedNumberFormatter unf,
+                                     UnicodeString skeleton,
+                                     const UnitInflectionTestCase *cases,
+                                     int32_t numCases,
+                                     IcuTestErrorCode &status);
 };
 
 class DecimalQuantityTest : public IntlTest {
@@ -181,13 +200,14 @@ class DecimalQuantityTest : public IntlTest {
     void testConvertToAccurateDouble();
     void testUseApproximateDoubleWhenAble();
     void testHardDoubleConversion();
+    void testFitsInLong();
     void testToDouble();
     void testMaxDigits();
     void testNickelRounding();
-    void testCompactDecimalSuppressedExponent();
+    void testScientificAndCompactSuppressedExponent();
     void testSuppressedExponentUnchangedByInitialScaling();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     void assertDoubleEquals(UnicodeString message, double a, double b);
@@ -200,7 +220,7 @@ class DoubleConversionTest : public IntlTest {
   public:
     void testDoubleConversionApi();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 };
 
 class ModifiersTest : public IntlTest {
@@ -210,7 +230,7 @@ class ModifiersTest : public IntlTest {
     void testSimpleModifier();
     void testCurrencySpacingEnabledModifier();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     void assertModifierEquals(const Modifier &mod, int32_t expectedPrefixLength, bool expectedStrong,
@@ -228,21 +248,22 @@ class PatternModifierTest : public IntlTest {
     void testPatternWithNoPlaceholder();
     void testMutableEqualsImmutable();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     UnicodeString getPrefix(const MutablePatternModifier &mod, UErrorCode &status);
     UnicodeString getSuffix(const MutablePatternModifier &mod, UErrorCode &status);
 };
 
-class PatternStringTest : public IntlTest {
+class PatternStringTest : public IntlTestWithFieldPosition {
   public:
     void testLocalized();
     void testToPatternSimple();
     void testExceptionOnInvalid();
     void testBug13117();
+    void testCurrencyDecimal();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
 };
@@ -259,7 +280,7 @@ class NumberParserTest : public IntlTest {
     void test20360_BidiOverflow();
     void testInfiniteRecursion();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 };
 
 class NumberSkeletonTest : public IntlTest {
@@ -274,8 +295,9 @@ class NumberSkeletonTest : public IntlTest {
     void flexibleSeparators();
     void wildcardCharacters();
     void perUnitInArabic();
+    void perUnitToSkeleton();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     void expectedErrorSkeleton(const char16_t** cases, int32_t casesLen);
@@ -291,16 +313,21 @@ class NumberRangeFormatterTest : public IntlTestWithFieldPosition {
     void testCollapse();
     void testIdentity();
     void testDifferentFormatters();
+    void testNaNInfinity();
     void testPlurals();
     void testFieldPositions();
     void testCopyMove();
     void toObject();
     void testGetDecimalNumbers();
+    void test21684_Performance();
+    void test21358_SignPosition();
+    void test21683_StateLeak();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 
   private:
     CurrencyUnit USD;
+    CurrencyUnit CHF;
     CurrencyUnit GBP;
     CurrencyUnit PTE;
 
@@ -336,7 +363,7 @@ class NumberPermutationTest : public IntlTest {
   public:
     void testPermutations();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override;
 };
 
 
@@ -354,7 +381,7 @@ class NumberPermutationTest : public IntlTest {
 
 class NumberTest : public IntlTest {
   public:
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) {
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = 0) override {
         if (exec) {
             logln("TestSuite NumberTest: ");
         }
