@@ -20,6 +20,7 @@
 
 #include <cstddef>
 #include <string.h>
+#include <limits>
 
 #include "unicode/utypes.h"
 #include "unicode/putil.h"
@@ -166,7 +167,7 @@ void
 StringTest::TestUpperOrdinal() {
     for (int32_t i = 0;; ++i) {
         char ic = nativeInvChars[i];
-        uint8_t ac = asciiInvChars[i];
+        uint8_t ac = static_cast<uint8_t>(asciiInvChars[i]);
         int32_t expected = ac - 'A';
         int32_t actual = uprv_upperOrdinal(ic);
         if (0 <= expected && expected <= 25) {
@@ -188,7 +189,7 @@ void
 StringTest::TestLowerOrdinal() {
     for (int32_t i = 0;; ++i) {
         char ic = nativeInvChars[i];
-        uint8_t ac = asciiInvChars[i];
+        uint8_t ac = static_cast<uint8_t>(asciiInvChars[i]);
         int32_t expected = ac - 'a';
         int32_t actual = uprv_lowerOrdinal(ic);
         if (0 <= expected && expected <= 25) {
@@ -243,15 +244,19 @@ void StringTest::runIndexedTest(int32_t index, UBool exec, const char *&name, ch
     TESTCASE_AUTO(TestSTLCompatibility);
     TESTCASE_AUTO(TestStringPiece);
     TESTCASE_AUTO(TestStringPieceComparisons);
+    TESTCASE_AUTO(TestStringPieceFind);
     TESTCASE_AUTO(TestStringPieceOther);
 #ifdef U_HAVE_STRING_VIEW
     TESTCASE_AUTO(TestStringPieceStringView);
 #endif
+    TESTCASE_AUTO(TestStringPieceU8);
     TESTCASE_AUTO(TestByteSink);
     TESTCASE_AUTO(TestCheckedArrayByteSink);
     TESTCASE_AUTO(TestStringByteSink);
+    TESTCASE_AUTO(TestStringByteSinkAppendU8);
     TESTCASE_AUTO(TestCharString);
     TESTCASE_AUTO(TestCStr);
+    TESTCASE_AUTO(TestCharStrAppendNumber);
     TESTCASE_AUTO(Testctou);
     TESTCASE_AUTO_END;
 }
@@ -264,7 +269,7 @@ StringTest::TestStringPiece() {
         errln("StringPiece() failed");
     }
     // Construct from NULL const char * pointer.
-    StringPiece null(NULL);
+    StringPiece null((const char *)nullptr);
     if(!null.empty() || null.data()!=NULL || null.length()!=0 || null.size()!=0) {
         errln("StringPiece(NULL) failed");
     }
@@ -394,7 +399,7 @@ StringTest::TestStringPiece() {
 void
 StringTest::TestStringPieceComparisons() {
     StringPiece empty;
-    StringPiece null(NULL);
+    StringPiece null(nullptr);
     StringPiece abc("abc");
     StringPiece abcd("abcdefg", 4);
     StringPiece abx("abx");
@@ -407,6 +412,35 @@ StringTest::TestStringPieceComparisons() {
     if(abc==abcd) {
         errln("abc==abcd");
     }
+
+    assertTrue("null<abc", null.compare(abc) < 0);
+    assertTrue("abc>null", abc.compare(null) > 0);
+    assertTrue("abc<abcd", abc.compare(abcd) < 0);
+    assertTrue("abcd>abc", abcd.compare(abc) > 0);
+    assertTrue("abc<abx", abc.compare(abx) < 0);
+    assertTrue("abx>abc", abx.compare(abc) > 0);
+    assertTrue("abx>abcd", abx.compare(abcd) > 0);
+    assertTrue("abcd<abx", abcd.compare(abx) < 0);
+    assertTrue("abx==abx", abx.compare(abx) == 0);
+
+    // Behavior should be the same as std::string::compare
+    {
+        std::string null("");
+        std::string abc("abc");
+        std::string abcd("abcdefg", 4);
+        std::string abx("abx");
+
+        assertTrue("std: null<abc", null.compare(abc) < 0);
+        assertTrue("std: abc>null", abc.compare(null) > 0);
+        assertTrue("std: abc<abcd", abc.compare(abcd) < 0);
+        assertTrue("std: abcd>abc", abcd.compare(abc) > 0);
+        assertTrue("std: abc<abx", abc.compare(abx) < 0);
+        assertTrue("std: abx>abc", abx.compare(abc) > 0);
+        assertTrue("std: abx>abcd", abx.compare(abcd) > 0);
+        assertTrue("std: abcd<abx", abcd.compare(abx) < 0);
+        assertTrue("std: abx==abx", abx.compare(abx) == 0);
+    }
+
     abcd.remove_suffix(1);
     if(abc!=abcd) {
         errln("abc!=abcd.remove_suffix(1)");
@@ -416,6 +450,51 @@ StringTest::TestStringPieceComparisons() {
     }
 }
 
+void
+StringTest::TestStringPieceFind() {
+    struct TestCase {
+        const char* haystack;
+        const char* needle;
+        int32_t expected;
+    } cases[] = {
+        { "", "", 0 },
+        { "", "x", -1 },
+        { "x", "", 0 },
+        { "x", "x", 0 },
+        { "xy", "x", 0 },
+        { "xy", "y", 1 },
+        { "xy", "xy", 0 },
+        { "xy", "xyz", -1 },
+        { "qwerty", "qqw", -1 },
+        { "qwerty", "qw", 0 },
+        { "qwerty", "er", 2 },
+        { "qwerty", "err", -1 },
+        { "qwerty", "ert", 2 },
+        { "qwerty", "ty", 4 },
+        { "qwerty", "tyy", -1 },
+        { "qwerty", "a", -1 },
+        { "qwerty", "abc", -1 }
+    };
+    int32_t caseNumber = 0;
+    for (auto& cas : cases) {
+        StringPiece haystack(cas.haystack);
+        StringPiece needle(cas.needle);
+        assertEquals(Int64ToUnicodeString(caseNumber),
+            cas.expected, haystack.find(needle, 0));
+        // Should be same as std::string::find
+        std::string stdhaystack(cas.haystack);
+        std::string stdneedle(cas.needle);
+        assertEquals(Int64ToUnicodeString(caseNumber) + u" (std)",
+            cas.expected, static_cast<int32_t>(stdhaystack.find(stdneedle, 0)));
+        // Test offsets against std::string::find
+        for (int32_t offset = 0; offset < haystack.length(); offset++) {
+            assertEquals(Int64ToUnicodeString(caseNumber) + "u @ " + Int64ToUnicodeString(offset),
+                static_cast<int32_t>(stdhaystack.find(stdneedle, offset)), haystack.find(needle, offset));
+        }
+        caseNumber++;
+    }
+}
+
 void
 StringTest::TestStringPieceOther() {
     static constexpr char msg[] = "Kapow!";
@@ -429,7 +508,7 @@ StringTest::TestStringPieceOther() {
     Other other;
     StringPiece piece(other);
 
-    assertEquals("size()", piece.size(), other.size());
+    assertEquals("size()", piece.size(), static_cast<int32_t>(other.size()));
     assertEquals("data()", piece.data(), other.data());
 }
 
@@ -446,18 +525,64 @@ StringTest::TestStringPieceStringView() {
 }
 #endif
 
+void
+StringTest::TestStringPieceU8() {
+    // ICU-20984 "mitigate some C++20 char8_t breakages"
+    // For the following APIs there are overloads for both
+    // const char * and const char8_t *.
+    // A u8"string literal" has one type or the other
+    // depending on C++ version and compiler settings.
+    StringPiece abc(u8"abc");
+    assertEquals("abc.length", 3, abc.length());
+    assertEquals("abc", "\x61\x62\x63", abc.data());
+
+    StringPiece abc3(u8"abcdef", 3);
+    assertEquals("abc3.length", 3, abc3.length());
+    assertEquals("abc3[0]", 0x61, abc3.data()[0]);
+    assertEquals("abc3[1]", 0x62, abc3.data()[1]);
+    assertEquals("abc3[2]", 0x63, abc3.data()[2]);
+
+    StringPiece uvw("q");
+    uvw.set(u8"uvw");
+    assertEquals("uvw.length", 3, uvw.length());
+    assertEquals("uvw", "\x75\x76\x77", uvw.data());
+
+    StringPiece xyz("r");
+    xyz.set(u8"xyzXYZ", 3);
+    assertEquals("xyz.length", 3, xyz.length());
+    assertEquals("xyz[0]", 0x78, xyz.data()[0]);
+    assertEquals("xyz[1]", 0x79, xyz.data()[1]);
+    assertEquals("xyz[2]", 0x7a, xyz.data()[2]);
+
+    StringPiece null(nullptr);
+    assertTrue("null is empty", null.empty());
+    assertTrue("null is null", null.data() == nullptr);
+
+#ifdef __cpp_lib_char8_t
+    std::u8string_view u8sv(u8"sv");  // C++20
+    StringPiece u8svsp(u8sv);
+    assertEquals("u8svsp.length", 2, u8svsp.length());
+    assertEquals("u8svsp", "\x73\x76", u8svsp.data());
+
+    std::u8string u8str(u8"str");  // C++20
+    StringPiece u8strsp(u8str);
+    assertEquals("u8strsp.length", 3, u8strsp.length());
+    assertEquals("u8strsp", "\x73\x74\x72", u8strsp.data());
+#endif  // __cpp_lib_char8_t
+}
+
 // Verify that ByteSink is subclassable and Flush() overridable.
 class SimpleByteSink : public ByteSink {
 public:
     SimpleByteSink(char *outbuf) : fOutbuf(outbuf), fLength(0) {}
-    virtual void Append(const char *bytes, int32_t n) {
+    virtual void Append(const char *bytes, int32_t n) override {
         if(fOutbuf != bytes) {
             memcpy(fOutbuf, bytes, n);
         }
         fOutbuf += n;
         fLength += n;
     }
-    virtual void Flush() { Append("z", 1); }
+    virtual void Flush() override { Append("z", 1); }
     int32_t length() { return fLength; }
 private:
     char *fOutbuf;
@@ -578,6 +703,20 @@ StringTest::TestStringByteSink() {
     }
 }
 
+void
+StringTest::TestStringByteSinkAppendU8() {
+    // ICU-20984 "mitigate some C++20 char8_t breakages"
+    // For the following APIs there are overloads for both
+    // const char * and const char8_t *.
+    // A u8"string literal" has one type or the other
+    // depending on C++ version and compiler settings.
+    std::string result("abc");
+    StringByteSink<std::string> sink(&result);
+    sink.AppendU8("def", 3);
+    sink.AppendU8(u8"ghijkl", 4);
+    assertEquals("abcdefghij", "abcdef\x67\x68\x69\x6a", result.c_str());
+}
+
 #if defined(_MSC_VER)
 #include <vector>
 #endif
@@ -673,6 +812,27 @@ StringTest::TestCharString() {
                 "Long string over 40 characters to trigger heap allocation",
                 s3.data());
     }
+
+    {
+        // extract()
+        errorCode.reset();
+        CharString s("abc", errorCode);
+        char buffer[10];
+
+        s.extract(buffer, 10, errorCode);
+        assertEquals("abc.extract(10) success", U_ZERO_ERROR, errorCode.get());
+        assertEquals("abc.extract(10) output", "abc", buffer);
+
+        strcpy(buffer, "012345");
+        s.extract(buffer, 3, errorCode);
+        assertEquals("abc.extract(3) not terminated",
+                     U_STRING_NOT_TERMINATED_WARNING, errorCode.reset());
+        assertEquals("abc.extract(3) output", "abc345", buffer);
+
+        strcpy(buffer, "012345");
+        s.extract(buffer, 2, errorCode);
+        assertEquals("abc.extract(2) overflow", U_BUFFER_OVERFLOW_ERROR, errorCode.reset());
+    }
 }
 
 void
@@ -684,6 +844,36 @@ StringTest::TestCStr() {
     }
 }
 
+void StringTest::TestCharStrAppendNumber() {
+    IcuTestErrorCode errorCode(*this, "TestCharStrAppendNumber()");
+
+    CharString testString;
+    testString.appendNumber(1, errorCode);
+    assertEquals("TestAppendNumber 1", "1", testString.data());
+
+    testString.clear();
+    testString.appendNumber(-1, errorCode);
+    assertEquals("TestAppendNumber -1", "-1", testString.data());
+
+    testString.clear();
+    testString.appendNumber(12345, errorCode);
+    assertEquals("TestAppendNumber 12345", "12345", testString.data());
+    testString.appendNumber(123, errorCode);
+    assertEquals("TestAppendNumber 12345 and then 123", "12345123", testString.data());
+
+    testString.clear();
+    testString.appendNumber(std::numeric_limits<int32_t>::max(), errorCode);
+    assertEquals("TestAppendNumber when appending the biggest int32", "2147483647", testString.data());
+
+    testString.clear();
+    testString.appendNumber(std::numeric_limits<int32_t>::min(), errorCode);
+    assertEquals("TestAppendNumber when appending the smallest int32", "-2147483648", testString.data());
+
+    testString.clear();
+    testString.appendNumber(0, errorCode);
+    assertEquals("TestAppendNumber when appending zero", "0", testString.data());
+}
+
 void
 StringTest::Testctou() {
   const char *cs = "Fa\\u0127mu";
