@@ -21,6 +21,7 @@
 #include "unicode/basictz.h"
 #include "unicode/tzfmt.h"
 #include "unicode/localpointer.h"
+#include "unicode/utf16.h"
 
 #include "cstring.h"
 #include "cstr.h"
@@ -83,7 +84,10 @@ TimeZoneFormatTest::runIndexedTest( int32_t index, UBool exec, const char* &name
         TESTCASE(4, TestFormat);
         TESTCASE(5, TestFormatTZDBNames);
         TESTCASE(6, TestFormatCustomZone);
-        default: name = ""; break;
+        TESTCASE(7, TestFormatTZDBNamesAllZoneCoverage);
+        TESTCASE(8, TestAdoptDefaultThreadSafe);
+        TESTCASE(9, TestCentralTime);
+    default: name = ""; break;
     }
 }
 
@@ -139,7 +143,9 @@ TimeZoneFormatTest::TestTimeZoneRoundTrip(void) {
         Locale("en"),
         Locale("en_CA"),
         Locale("fr"),
-        Locale("zh_Hant")
+        Locale("zh_Hant"),
+        Locale("fa"),
+        Locale("ccp")
     };
 
     const Locale *LOCALES;
@@ -152,7 +158,11 @@ TimeZoneFormatTest::TestTimeZoneRoundTrip(void) {
         LOCALES = Locale::getAvailableLocales(nLocales);
     }
 
-    StringEnumeration *tzids = TimeZone::createEnumeration();
+    StringEnumeration *tzids = TimeZone::createEnumeration(status);
+    if (U_FAILURE(status)) {
+        dataerrln("Unable to create TimeZone enumeration");
+        return;
+    }
     int32_t inRaw, inDst;
     int32_t outRaw, outDst;
 
@@ -168,7 +178,6 @@ TimeZoneFormatTest::TestTimeZoneRoundTrip(void) {
         gmtFmt.format(0.0, localGMTString);
 
         for (int32_t patidx = 0; patidx < UPRV_LENGTHOF(PATTERNS); patidx++) {
-
             SimpleDateFormat *sdf = new SimpleDateFormat((UnicodeString)PATTERNS[patidx], LOCALES[locidx], status);
             if (U_FAILURE(status)) {
                 dataerrln((UnicodeString)"new SimpleDateFormat failed for pattern " +
@@ -228,7 +237,7 @@ TimeZoneFormatTest::TestTimeZoneRoundTrip(void) {
                         UnicodeString canonicalID;
                         TimeZone::getCanonicalID(*tzid, canonicalID, status);
                         if (U_FAILURE(status)) {
-                            // Uknown ID - we should not get here
+                            // Unknown ID - we should not get here
                             errln((UnicodeString)"Unknown ID " + *tzid);
                             status = U_ZERO_ERROR;
                         } else if (outtzid != canonicalID) {
@@ -261,7 +270,7 @@ TimeZoneFormatTest::TestTimeZoneRoundTrip(void) {
                         UnicodeString canonical;
                         TimeZone::getCanonicalID(*tzid, canonical, status);
                         if (U_FAILURE(status)) {
-                            // Uknown ID - we should not get here
+                            // Unknown ID - we should not get here
                             errln((UnicodeString)"Unknown ID " + *tzid);
                             status = U_ZERO_ERROR;
                         } else if (outtzid != canonical) {
@@ -299,10 +308,13 @@ TimeZoneFormatTest::TestTimeZoneRoundTrip(void) {
                         if (!isOffsetFormat) {
                             // Check if localized GMT format is used as a fallback of name styles
                             int32_t numDigits = 0;
-                            for (int n = 0; n < tzstr.length(); n++) {
-                                if (u_isdigit(tzstr.charAt(n))) {
+                            int32_t idx = 0;
+                            while (idx < tzstr.length()) {
+                                UChar32 cp = tzstr.char32At(idx);
+                                if (u_isdigit(cp)) {
                                     numDigits++;
                                 }
+                                idx += U16_LENGTH(cp);
                             }
                             isOffsetFormat = (numDigits > 0);
                         }
@@ -403,7 +415,7 @@ struct LocaleData {
         for (int i=0; i<UPRV_LENGTHOF(times); i++) {
             times[i] = 0;
         }
-    };
+    }
 
     void resetTestIteration() {
         localeIndex = -1;
@@ -478,7 +490,7 @@ TimeZoneFormatTest::TestTimeRoundTrip(void) {
         Locale("ko_KR"), Locale("nb_NO"), Locale("nl_NL"), Locale("nn_NO"), Locale("pl_PL"),
         Locale("pt"), Locale("pt_BR"), Locale("pt_PT"), Locale("ru_RU"), Locale("sv_SE"),
         Locale("th_TH"), Locale("tr_TR"), Locale("zh"), Locale("zh_Hans"), Locale("zh_Hans_CN"),
-        Locale("zh_Hant"), Locale("zh_Hant_TW")
+        Locale("zh_Hant"), Locale("zh_Hant_TW"), Locale("fa"), Locale("ccp")
     };
 
     if (bTestAll) {
@@ -595,7 +607,8 @@ void TimeZoneFormatTest::RunTimeRoundTripTests(int32_t threadNumber) {
                 }
             }
 
-            if (*tzid == "Pacific/Apia" && uprv_strcmp(PATTERNS[patidx], "vvvv") == 0
+            if ((*tzid == "Pacific/Apia" || *tzid == "Pacific/Midway" || *tzid == "Pacific/Pago_Pago")
+                    && uprv_strcmp(PATTERNS[patidx], "vvvv") == 0
                     && logKnownIssue("11052", "Ambiguous zone name - Samoa Time")) {
                 continue;
             }
@@ -704,6 +717,47 @@ void TimeZoneFormatTest::RunTimeRoundTripTests(int32_t threadNumber) {
     delete tzids;
 }
 
+void
+TimeZoneFormatTest::TestAdoptDefaultThreadSafe(void) {
+    ThreadPool<TimeZoneFormatTest> threads(this, threadCount, &TimeZoneFormatTest::RunAdoptDefaultThreadSafeTests);
+    threads.start();   // Start all threads.
+    threads.join();    // Wait for all threads to finish.
+}
+
+static const int32_t kAdoptDefaultIteration = 10;
+static const int32_t kCreateDefaultIteration = 5000;
+static const int64_t kStartTime = 1557288964845;
+
+void TimeZoneFormatTest::RunAdoptDefaultThreadSafeTests(int32_t threadNumber) {
+    UErrorCode status = U_ZERO_ERROR;
+    if (threadNumber % 2 == 0) {
+        for (int32_t i = 0; i < kAdoptDefaultIteration; i++) {
+            std::unique_ptr<icu::StringEnumeration> timezones(
+                    icu::TimeZone::createEnumeration(status));
+            // Fails with missing data.
+            if (U_FAILURE(status)) {
+                dataerrln("Unable to create TimeZone enumeration");
+                return;
+            }
+            while (const icu::UnicodeString* timezone = timezones->snext(status)) {
+                status = U_ZERO_ERROR;
+                icu::TimeZone::adoptDefault(icu::TimeZone::createTimeZone(*timezone));
+            }
+        }
+    } else {
+        int32_t rawOffset;
+        int32_t dstOffset;
+        int64_t date = kStartTime;
+        for (int32_t i = 0; i < kCreateDefaultIteration; i++) {
+            date += 6000 * i;
+            std::unique_ptr<icu::TimeZone> tz(icu::TimeZone::createDefault());
+            status = U_ZERO_ERROR;
+            tz->getOffset(static_cast<UDate>(date), TRUE, rawOffset, dstOffset, status);
+            status = U_ZERO_ERROR;
+            tz->getOffset(static_cast<UDate>(date), FALSE, rawOffset, dstOffset, status);
+        }
+    }
+}
 
 typedef struct {
     const char*     text;
@@ -1251,5 +1305,100 @@ TimeZoneFormatTest::TestFormatCustomZone(void) {
     }
 }
 
+void
+TimeZoneFormatTest::TestFormatTZDBNamesAllZoneCoverage(void) {
+    UErrorCode status = U_ZERO_ERROR;
+    LocalPointer<StringEnumeration> tzids(TimeZone::createEnumeration(status));
+    if (U_FAILURE(status)) {
+        dataerrln("Unable to create TimeZone enumeration", __FILE__, __LINE__);
+        return;
+    }
+    const UnicodeString *tzid;
+    LocalPointer<TimeZoneNames> tzdbNames(TimeZoneNames::createTZDBInstance(Locale("en"), status));
+    UDate now = Calendar::getNow();
+    UnicodeString mzId;
+    UnicodeString name;
+    while ((tzid = tzids->snext(status))) {
+        logln("Zone: " + *tzid);
+        LocalPointer<TimeZone> tz(TimeZone::createTimeZone(*tzid));
+        tzdbNames->getMetaZoneID(*tzid, now, mzId);
+        if (mzId.isBogus()) {
+            logln((UnicodeString)"Meta zone: <not available>");
+        } else {
+            logln((UnicodeString)"Meta zone: " + mzId);
+        }
 
+        // mzID could be bogus here
+        tzdbNames->getMetaZoneDisplayName(mzId, UTZNM_SHORT_STANDARD, name);
+        // name could be bogus here
+        if (name.isBogus()) {
+            logln((UnicodeString)"Meta zone short standard name: <not available>");
+        }
+        else {
+            logln((UnicodeString)"Meta zone short standard name: " + name);
+        }
+
+        tzdbNames->getMetaZoneDisplayName(mzId, UTZNM_SHORT_DAYLIGHT, name);
+        // name could be bogus here
+        if (name.isBogus()) {
+            logln((UnicodeString)"Meta zone short daylight name: <not available>");
+        }
+        else {
+            logln((UnicodeString)"Meta zone short daylight name: " + name);
+        }
+    }
+}
+
+// Test for checking parse results are same for a same input string
+// using SimpleDateFormat initialized with different regional locales - US and Belize.
+// Belize did not observe DST from 1968 to 1973, 1975 to 1982, and 1985 and later.
+void
+TimeZoneFormatTest::TestCentralTime(void) {
+    UnicodeString pattern(u"y-MM-dd HH:mm:ss zzzz");
+    UnicodeString testInputs[] = {
+        // 1970-01-01 - Chicago:STD/Belize:STD
+        u"1970-01-01 12:00:00 Central Standard Time",
+        u"1970-01-01 12:00:00 Central Daylight Time",
+
+        // 1970-07-01 - Chicago:STD/Belize:STD
+        u"1970-07-01 12:00:00 Central Standard Time",
+        u"1970-07-01 12:00:00 Central Daylight Time",
+
+        // 1974-01-01 - Chicago:STD/Belize:DST
+        u"1974-01-01 12:00:00 Central Standard Time",
+        u"1974-01-01 12:00:00 Central Daylight Time",
+
+        // 2020-01-01 - Chicago:STD/Belize:STD
+        u"2020-01-01 12:00:00 Central Standard Time",
+        u"2020-01-01 12:00:00 Central Daylight Time",
+
+        // 2020-01-01 - Chicago:DST/Belize:STD
+        u"2020-07-01 12:00:00 Central Standard Time",
+        u"2020-07-01 12:00:00 Central Daylight Time",
+
+        u""
+    };
+
+    UErrorCode status = U_ZERO_ERROR;
+    SimpleDateFormat sdfUS(pattern, Locale("en_US"), status);
+    SimpleDateFormat sdfBZ(pattern, Locale("en_BZ"), status);
+    if (U_FAILURE(status)) {
+        errln("Failed to create SimpleDateFormat instance");
+        return;
+    }
+
+    for (int32_t i = 0; !testInputs[i].isEmpty(); i++) {
+        UDate dUS = sdfUS.parse(testInputs[i], status);
+        UDate dBZ = sdfBZ.parse(testInputs[i], status);
+
+        if (U_FAILURE(status)) {
+            errln((UnicodeString)"Failed to parse date string: " + testInputs[i]);
+            continue;
+        }
+
+        if (dUS != dBZ) {
+            errln((UnicodeString)"Parse results should be same for input: " + testInputs[i]);
+        }
+    }
+}
 #endif /* #if !UCONFIG_NO_FORMATTING */
