@@ -1,4 +1,4 @@
-// Copyright (C) 2016 and later: Unicode, Inc. and others.
+// © 2016 and later: Unicode, Inc. and others.
 // License & terms of use: http://www.unicode.org/copyright.html
 /*
 ********************************************************************************
@@ -13,7 +13,7 @@
 
 #include "unicode/utypes.h"
 
-#if U_PLATFORM_HAS_WIN32_API
+#if U_PLATFORM_USES_ONLY_WIN32_API
 
 #if !UCONFIG_NO_FORMATTING
 
@@ -28,6 +28,8 @@
 #include "winutil.h"
 #include "windttst.h"
 
+#include "dtfmttst.h"
+
 #include "cmemory.h"
 #include "cstring.h"
 #include "locmap.h"
@@ -41,6 +43,8 @@
 #   define NOMCX
 #   include <windows.h>
 
+#include <algorithm>
+
 static const char *getCalendarType(int32_t type)
 {
     switch (type)
@@ -66,7 +70,7 @@ static const char *getCalendarType(int32_t type)
     }
 }
 
-void Win32DateTimeTest::testLocales(TestLog *log)
+void Win32DateTimeTest::testLocales(DateFormatTest *log)
 {
     SYSTEMTIME winNow;
     UDate icuNow = 0;
@@ -84,7 +88,8 @@ void Win32DateTimeTest::testLocales(TestLog *log)
         for (int z = 0; z < ec; z += 1) {
             UnicodeString equiv = TimeZone::getEquivalentID(zoneID, z);
 
-            if (found = uprv_getWindowsTimeZoneInfo(&tzi, equiv.getBuffer(), equiv.length())) {
+            found = uprv_getWindowsTimeZoneInfo(&tzi, equiv.getBuffer(), equiv.length());
+            if (found) {
                 break;
             }
         }
@@ -111,6 +116,7 @@ void Win32DateTimeTest::testLocales(TestLog *log)
     for(int i = 0; i < lcidCount; i += 1) {
         UErrorCode status = U_ZERO_ERROR;
         WCHAR longDateFormat[81], longTimeFormat[81], wdBuffer[256], wtBuffer[256];
+        DWORD value = 0;
         int32_t calType = 0;
 
         // NULL localeID means ICU didn't recognize this locale
@@ -118,10 +124,26 @@ void Win32DateTimeTest::testLocales(TestLog *log)
             continue;
         }
 
+        // Some locales have had their names change over various OS releases; skip them in the test for now.
+        int32_t failingLocaleLCIDs[] = {
+            0x040a, /* es-ES_tradnl;es-ES-u-co-trad; */
+            0x048c, /* fa-AF;prs-AF;prs-Arab-AF; */
+            0x046b, /* qu-BO;quz-BO;quz-Latn-BO; */
+            0x086b, /* qu-EC;quz-EC;quz-Latn-EC; */
+            0x0c6b, /* qu-PE;quz-PE;quz-Latn-PE; */
+            0x0492  /* ckb-IQ;ku-Arab-IQ; */
+        };
+        bool skip = (std::find(std::begin(failingLocaleLCIDs), std::end(failingLocaleLCIDs), lcidRecords[i].lcid) != std::end(failingLocaleLCIDs));
+        if (skip && log->logKnownIssue("13119", "Windows '@compat=host' fails on down-level versions of the OS")) {
+            log->logln("ticket:13119 - Skipping LCID = 0x%04x", lcidRecords[i].lcid);
+            continue;
+        }
+
         GetLocaleInfoW(lcidRecords[i].lcid, LOCALE_SLONGDATE,   longDateFormat, 81);
         GetLocaleInfoW(lcidRecords[i].lcid, LOCALE_STIMEFORMAT, longTimeFormat, 81);
-        GetLocaleInfoW(lcidRecords[i].lcid, LOCALE_RETURN_NUMBER|LOCALE_ICALENDARTYPE, (LPWSTR) calType, sizeof(int32_t));
+        GetLocaleInfoW(lcidRecords[i].lcid, LOCALE_RETURN_NUMBER|LOCALE_ICALENDARTYPE, (LPWSTR)&value, sizeof(value)/sizeof(WCHAR));
 
+        calType = value;
         char localeID[64];
 
         uprv_strcpy(localeID, lcidRecords[i].localeID);
@@ -134,7 +156,7 @@ void Win32DateTimeTest::testLocales(TestLog *log)
         wdLength = GetDateFormatW(lcidRecords[i].lcid, DATE_LONGDATE, &winNow, NULL, wdBuffer, UPRV_LENGTHOF(wdBuffer));
         wtLength = GetTimeFormatW(lcidRecords[i].lcid, 0, &winNow, NULL, wtBuffer, UPRV_LENGTHOF(wtBuffer));
 
-        if (uprv_strchr(localeID, '@') > 0) {
+        if (uprv_strchr(localeID, '@')) {
             uprv_strcat(localeID, ";");
         } else {
             uprv_strcat(localeID, "@");
@@ -193,4 +215,4 @@ void Win32DateTimeTest::testLocales(TestLog *log)
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
-#endif /* U_PLATFORM_HAS_WIN32_API */
+#endif /* U_PLATFORM_USES_ONLY_WIN32_API */
