@@ -1,7 +1,9 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  *
- *   Copyright (C) 1999-2008, International Business Machines
+ *   Copyright (C) 1999-2014, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *
  *******************************************************************************
@@ -23,7 +25,9 @@
 #include "layout/LayoutEngine.h"
 #include "layout/LELanguages.h"
 
+#ifndef USING_ICULEHB
 #include "OpenTypeLayoutEngine.h"
+#endif
 
 #include "letest.h"
 
